/*****************************************************************************
* | File      	:   DEV_Config.c
* | Author      :   Waveshare team
* | Function    :   Hardware underlying interface
* | Info        :
*----------------
* |	This version:   V2.1
* | Date        :   2022-01-22
* | Info        :   Basic version
*
******************************************************************************/
#include "DEV_Config.h"
#include <unistd.h>
#include <fcntl.h>

uint32_t UART_fd;
char *UART_Device;

int TXDEN_0;
int TXDEN_1;
int TXDEN_2;
int TXDEN_3;

/******************************************************************************
function:	GPIO Function initialization and transfer
parameter:
Info:
******************************************************************************/
void DEV_GPIO_Mode(uint16_t Pin, uint16_t Mode)
{
    /*
        0:  INPT   
        1:  OUTP
    */
#ifdef USE_BCM2835_LIB  
    if(Mode == 0 || Mode == BCM2835_GPIO_FSEL_INPT){
        bcm2835_gpio_fsel(Pin, BCM2835_GPIO_FSEL_INPT);
    }else {
        bcm2835_gpio_fsel(Pin, BCM2835_GPIO_FSEL_OUTP);
    }
#elif USE_WIRINGPI_LIB
    if(Mode == 0 || Mode == INPUT){
        pinMode(Pin, INPUT);
        pullUpDnControl(Pin, PUD_UP);
    }else{ 
        pinMode(Pin, OUTPUT);
    }
#elif USE_DEV_LIB
    SYSFS_GPIO_Export(Pin);
    if(Mode == 0 || Mode == SYSFS_GPIO_IN){
        SYSFS_GPIO_Direction(Pin, SYSFS_GPIO_IN);
    }else{
        SYSFS_GPIO_Direction(Pin, SYSFS_GPIO_OUT);
    }
#endif   
}

void DEV_Digital_Write(uint16_t Pin, uint8_t Value)
{
#ifdef USE_BCM2835_LIB
    bcm2835_gpio_write(Pin, Value);
    
#elif USE_WIRINGPI_LIB
    digitalWrite(Pin, Value);
    
#elif USE_DEV_LIB
    SYSFS_GPIO_Write(Pin, Value);
    
#endif
}

uint8_t DEV_Digital_Read(uint16_t Pin)
{
    uint8_t Read_value = 0;
#ifdef USE_BCM2835_LIB
    Read_value = bcm2835_gpio_lev(Pin);
    
#elif USE_WIRINGPI_LIB
    Read_value = digitalRead(Pin);
    
#elif USE_DEV_LIB
    Read_value = SYSFS_GPIO_Read(Pin);
#endif
    return Read_value;
}


/**
 * delay x ms
**/
void DEV_Delay_ms(UDOUBLE xms)
{
#ifdef USE_BCM2835_LIB
    bcm2835_delay(xms);
#elif USE_WIRINGPI_LIB
    delay(xms);
#elif USE_DEV_LIB
    UDOUBLE i;
    for(i=0; i < xms; i++){
        usleep(1000);
    }
#endif
}


void GPIO_Config(void)
{
	TXDEN_0 = 10;
	TXDEN_1 = 27;
	TXDEN_2 = 21;
	TXDEN_3 = 7;
    
	DEV_GPIO_Mode(TXDEN_0, 1);
    DEV_GPIO_Mode(TXDEN_1, 1);
    DEV_GPIO_Mode(TXDEN_2, 1);
	DEV_GPIO_Mode(TXDEN_3, 1);
	
	DEV_Digital_Write(TXDEN_0, 1);
	DEV_Digital_Write(TXDEN_1, 1);
	DEV_Digital_Write(TXDEN_2, 1);
	DEV_Digital_Write(TXDEN_3, 1);
}

/******************************************************************************
function:	UART Function initialization and transfer
parameter:
Info:
******************************************************************************/
void DEV_UART_Init(char *Device)
{
    UART_Device = Device;

#ifdef USE_BCM2835_LIB
	printf("BCM2835 Not Supported UART!!! \r\n");
	
#elif USE_WIRINGPI_LIB
	if((UART_fd = serialOpen(UART_Device, 115200)) < 0){
		printf("set uart failed  !!! \r\n");
		return ;
	}else {
		printf("set uart success  !!! \r\n");
	}
#elif USE_DEV_LIB
	DEV_HARDWARE_UART_begin(UART_Device);
	UART_Set_Baudrate(115200);
#endif
    
}

void UART_Set_Baudrate(uint32_t Baudrate)
{

#ifdef USE_BCM2835_LIB
	printf("BCM2835 Not Supported UART!!! \r\n");
	
#elif USE_WIRINGPI_LIB
	serialClose(UART_fd);
	if((UART_fd = serialOpen(UART_Device, Baudrate)) < 0){
		printf("set uart failed	!!! \r\n");
	}else{
		printf("set uart success  !!! \r\n");
	}
	
#elif USE_DEV_LIB
	DEV_HARDWARE_UART_setBaudrate(Baudrate);
#endif

}

void UART_Write_Byte(uint8_t data)
{
#ifdef USE_BCM2835_LIB
	printf("BCM2835 Not Supported UART!!! \r\n");
	
#elif USE_WIRINGPI_LIB
	serialPutchar(UART_fd, data);
	
#elif USE_DEV_LIB
	DEV_HARDWARE_UART_writeByte(data);
#endif
}

int UART_Read_Byte(void)
{
	int ref;
	
#ifdef USE_BCM2835_LIB
	printf("BCM2835 Not Supported UART!!! \r\n");
	
#elif USE_WIRINGPI_LIB
	ref = serialGetchar(UART_fd);
	
#elif USE_DEV_LIB
	ref = DEV_HARDWARE_UART_readByte();
#endif

	return ref;
}


int UART_Write_nByte(uint8_t *pData, uint32_t Lan)
{
#ifdef USE_BCM2835_LIB
	printf("BCM2835 Not Supported UART!!! \r\n");
	
#elif USE_WIRINGPI_LIB
	for(uint32_t i=0; i<Lan; i++){
		serialPutchar(UART_fd, *(pData+i));
	}
	
	
#elif USE_DEV_LIB
	DEV_HARDWARE_UART_write(pData, Lan);
#endif
}
/******************************************************************************
function:	Module Initialize, the library and initialize the pins, SPI protocol
parameter:
Info:
******************************************************************************/
uint8_t DEV_ModuleInit(void)
{
 #ifdef USE_BCM2835_LIB
    if(!bcm2835_init()) {
        printf("bcm2835 init failed  !!! \r\n");
        return 1;
    } else {
        printf("bcm2835 init success !!! \r\n");
    }

#elif USE_WIRINGPI_LIB  
    //if(wiringPiSetup() < 0)//use wiringpi Pin number table  
    if(wiringPiSetupGpio() < 0) { //use BCM2835 Pin number table
        printf("set wiringPi lib failed	!!! \r\n");
        return 1;
    } else {
        printf("set wiringPi lib success  !!! \r\n");
    }

#elif USE_DEV_LIB
    printf("USE_DEV_LIB \r\n");
#endif
    GPIO_Config();

    return 0;
}

/******************************************************************************
function:	Module exits, closes SPI and BCM2835 library
parameter:
Info:
******************************************************************************/
void DEV_ModuleExit(void)
{
#ifdef USE_BCM2835_LIB
    bcm2835_close();
    
#elif USE_WIRINGPI_LIB
	serialFlush(UART_fd);
	serialClose(UART_fd);
    
#elif USE_DEV_LIB

#endif
}

